@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -45,7 +45,7 @@ tp_new (PyTypeObject *type, PyObject *, PyObject *)
       catch (const std::runtime_error& e)
         {
           Py_DECREF (ret);
-          PyErr_SetString (PyExc_IOError, e.what ());
+          mobius::py::set_io_error (e.what ());
           ret = nullptr;
         }
     }
@@ -54,7 +54,7 @@ tp_new (PyTypeObject *type, PyObject *, PyObject *)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief application: tp_dealloc
+//! \brief Application: tp_dealloc
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static void
 tp_dealloc (core_application_o *self)
@@ -64,39 +64,39 @@ tp_dealloc (core_application_o *self)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief application: name getter
+//! \brief Application: name getter
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
 tp_getter_name (core_application_o *self)
 {
-  return PyString2_from_std_string (self->obj->get_name ());
+  return mobius::py::pystring_from_std_string (self->obj->get_name ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief application: version getter
+//! \brief Application: version getter
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
 tp_getter_version (core_application_o *self)
 {
-  return PyString2_from_std_string (self->obj->get_version ());
+  return mobius::py::pystring_from_std_string (self->obj->get_version ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief application: title getter
+//! \brief Application: title getter
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
 tp_getter_title (core_application_o *self)
 {
-  return PyString2_from_std_string (self->obj->get_title ());
+  return mobius::py::pystring_from_std_string (self->obj->get_title ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief application: copyright getter
+//! \brief Application: copyright getter
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
 tp_getter_copyright (core_application_o *self)
 {
-  return PyString2_from_std_string (self->obj->get_copyright ());
+  return mobius::py::pystring_from_std_string (self->obj->get_copyright ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -132,7 +132,7 @@ static PyGetSetDef tp_getset[] =
 };
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief application: has_config
+//! \brief Application: has_config
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
 tp_f_has_config (core_application_o *self, PyObject *args)
@@ -146,7 +146,7 @@ tp_f_has_config (core_application_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -159,7 +159,7 @@ tp_f_has_config (core_application_o *self, PyObject *args)
     }
   catch (const std::runtime_error& e)
     {
-      PyErr_SetString (PyExc_IOError, e.what ());
+      mobius::py::set_io_error (e.what ());
     }
     
   return ret;
@@ -180,7 +180,7 @@ tp_f_get_config (core_application_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -193,7 +193,7 @@ tp_f_get_config (core_application_o *self, PyObject *args)
     }
   catch (const std::runtime_error& e)
     {
-      PyErr_SetString (PyExc_IOError, e.what ());
+      mobius::py::set_io_error (e.what ());
     }
     
   return ret;
@@ -216,7 +216,7 @@ tp_f_set_config (core_application_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -238,7 +238,7 @@ tp_f_set_config (core_application_o *self, PyObject *args)
     }
   catch (const std::runtime_error& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -259,7 +259,7 @@ tp_f_get_config_path (core_application_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -272,7 +272,7 @@ tp_f_get_config_path (core_application_o *self, PyObject *args)
     }
   catch (const std::runtime_error& e)
     {
-      PyErr_SetString (PyExc_IOError, e.what ());
+      mobius::py::set_io_error (e.what ());
     }
     
   return ret;
@@ -293,7 +293,7 @@ tp_f_get_cache_path (core_application_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -306,7 +306,7 @@ tp_f_get_cache_path (core_application_o *self, PyObject *args)
     }
   catch (const std::runtime_error& e)
     {
-      PyErr_SetString (PyExc_IOError, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
     
   return ret;
@@ -327,7 +327,7 @@ tp_f_get_data_path (core_application_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -340,7 +340,51 @@ tp_f_get_data_path (core_application_o *self, PyObject *args)
     }
   catch (const std::runtime_error& e)
     {
-      PyErr_SetString (PyExc_IOError, e.what ());
+      mobius::py::set_runtime_error (e.what ());
+    }
+    
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief application: start
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_start (core_application_o *self, PyObject *)
+{
+  // execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      self->obj->start ();
+      ret = mobius::py::pynone ();
+    }
+  catch (const std::runtime_error& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+    
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief application: stop
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_stop (core_application_o *self, PyObject *)
+{
+  // execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      self->obj->stop ();
+      ret = mobius::py::pynone ();
+    }
+  catch (const std::runtime_error& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
     }
     
   return ret;
@@ -360,11 +404,11 @@ tp_f_new_transaction (core_application_o *self, PyObject *)
 
   try
     {
-      ret = PyMobius_Database_Transaction_from_cpp (self->obj->new_transaction ());
+      ret = pymobius_database_transaction_to_pyobject (self->obj->new_transaction ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -412,6 +456,18 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "get data path"
   },
+  {
+    (char *) "start",
+    (PyCFunction) tp_f_start,
+    METH_VARARGS,
+    "start application"
+  },
+  {
+    (char *) "stop",
+    (PyCFunction) tp_f_stop,
+    METH_VARARGS,
+    "stop application"
+  },
   {
     (char *) "new_transaction",
     (PyCFunction) tp_f_new_transaction,
