@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -113,6 +113,7 @@ EMOJI_CHARS = {
   'vampire'              : '🧛',
   'victory'              : '✌',
   'wasntme'              : '🙄',
+  'wave'                 : '🌊',
   'wink'                 : '😉',
   'worry'                : '😟',
   'xd'                   : '😆',
@@ -442,6 +443,25 @@ class MessageParser (object):
 
     return self.__elements
 
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Parse notice
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def parse_notice (self):
+    l = json.loads (self.__text)
+
+    if len (l) > 0:
+      c = l[0]
+      text = ''
+
+      for at in c.get ('attachments', []):
+        text += at.get ('content', {}).get (u'text')
+
+      if text:
+        element = {u'type' : u'system', u'text' : text}
+        self.add_element (element)
+
+    return self.__elements
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse popcard
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -645,6 +665,8 @@ class MessageParser (object):
       mobius.core.logf ('WRN app.skype: invalid <partlist> ' + text)
       return
 
+    partlist_type = e.attributes.get (u'type')
+
     # get all <part> nodes
     e = self.__parser.get ()
 
@@ -663,7 +685,16 @@ class MessageParser (object):
       e = self.__parser.get ()
 
     # create element
-    text = u'Participants: ' + u', '.join (account + u' (' + name + u')' if name else account for (account, name) in sorted (parts))
+    if partlist_type == 'ended':
+      text = u'Call ended.'
+
+    elif partlist_type == 'started':
+      text = u'Call started.'
+
+    else:
+      mobius.core.logf ('DEV unknown partlist type "%s"' % partlist_type)
+
+    text += u' Participants: ' + u', '.join (account + u' (' + name + u')' if name else account for (account, name) in sorted (parts))
 
     element = {u'type' : u'system', u'text' : text}
     self.add_element (element)
@@ -891,6 +922,49 @@ class MessageParser (object):
     element = {u'type' : u'system', u'text' : text}
     self.add_element (element)
 
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Parse <addmember> tag
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def _parse_addmember (self, e):
+    timestamp = None
+    initiator = None
+    target = None
+
+    # process until </addmember>
+    e = self.__parser.get ()
+
+    while e.type != 0 and (e.type != 3 or e.text != 'addmember'):
+      etext = unicode (e.text, 'utf-8')
+      
+      if e.type == 2:		# <tag>
+
+        if etext == u'eventtime':
+          s = int (self.__get_tag_value (e)) // 1000
+          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (s)
+
+        elif etext == u'initiator':
+          initiator = format_account_id (self.__get_tag_value (e))
+
+        elif etext == u'target':
+          target = format_account_id (self.__get_tag_value (e))
+
+      e = self.__parser.get ()
+
+    # create element
+    text = u'Member "%s"' % target
+    
+    if initiator and initiator != target:
+      text += u' added to chat by user "%s"' % initiator
+
+    else:
+      text += u' entered chat'
+
+    if timestamp:
+      text += u' at %s' % timestamp
+
+    element = {u'type' : u'system', u'text' : text}
+    self.add_element (element)
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse <deletemember> tag
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -908,8 +982,8 @@ class MessageParser (object):
       if e.type == 2:		# <tag>
 
         if etext == u'eventtime':
-          value = self.__get_tag_value (e)
-          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (int (value) // 1000)
+          s = int (self.__get_tag_value (e)) // 1000
+          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (s)
 
         elif etext == u'initiator':
           initiator = format_account_id (self.__get_tag_value (e))
@@ -920,13 +994,100 @@ class MessageParser (object):
       e = self.__parser.get ()
 
     # create element
-    text = u'Member "%s" deleted from chat' % target
+    text = u'Member "%s"' % target
     
-    if initiator:
-      text += u' by user "%s"' % initiator
-      
+    if initiator and initiator != target:
+      text += u' removed from chat by user "%s"' % initiator
+
+    else:
+      text += u' left chat'
+
     if timestamp:
-      text += u' in %s' % timestamp
+      text += u' at %s' % timestamp
+
+    element = {u'type' : u'system', u'text' : text}
+    self.add_element (element)
+
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Parse <topicupdate> tag
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def _parse_topicupdate (self, e):
+    timestamp = None
+    initiator = None
+    value = None
+
+    # process until </topicupdate>
+    e = self.__parser.get ()
+
+    while e.type != 0 and (e.type != 3 or e.text != 'topicupdate'):
+      etext = unicode (e.text, 'utf-8')
+      
+      if e.type == 2:		# <tag>
+
+        if etext == u'eventtime':
+          s = int (self.__get_tag_value (e)) // 1000
+          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (s)
+
+        elif etext == u'initiator':
+          initiator = format_account_id (self.__get_tag_value (e))
+
+        elif etext == u'value':
+          value = self.__get_tag_value (e)
+
+      e = self.__parser.get ()
+
+    # create element
+    text = u'Topic updated to "%s"' % value
+    
+    if initiator:
+      text += u'  by user "%s"' % initiator
+
+    if timestamp:
+      text += u' at %s' % timestamp
+
+    element = {u'type' : u'system', u'text' : text}
+    self.add_element (element)
+    
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Parse <historydisclosedupdate> tag
+  # @see https://docs.microsoft.com/en-us/javascript/api/botbuilder/iconversationupdate?view=botbuilder-ts-3.0#botbuilder-iconversationupdate-historydisclosed
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def _parse_historydisclosedupdate (self, e):
+    timestamp = None
+    initiator = None
+    value = None
+
+    # process until </historydisclosedupdate>
+    e = self.__parser.get ()
+
+    while e.type != 0 and (e.type != 3 or e.text != 'historydisclosedupdate'):
+      etext = unicode (e.text, 'utf-8')
+      
+      if e.type == 2:		# <tag>
+
+        if etext == u'eventtime':
+          s = int (self.__get_tag_value (e)) // 1000
+          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (s)
+
+        elif etext == u'initiator':
+          initiator = format_account_id (self.__get_tag_value (e))
+
+        elif etext == u'value':
+          value = self.__get_tag_value (e)
+
+      e = self.__parser.get ()
+
+    # create element
+    if value == "True":
+      text = u'History disclosed'
+    else:
+      text = u'History closed'
+    
+    if initiator:
+      text += u'  by user "%s"' % initiator
+
+    if timestamp:
+      text += u' at %s' % timestamp
 
     element = {u'type' : u'system', u'text' : text}
     self.add_element (element)
